#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Mon Aug 31 19:24:28 2020

@author: finazzi
"""

# Cargamos la librerías que vamos a necesitar
import matplotlib.pyplot as plt

from scipy import ndimage   # Para rotar imagenes
from scipy import signal    # Para aplicar filtros
import numpy as np          # Para porcesar matrices y vectores


from scipy.signal import savgol_filter as savgol


#%%




def medir_angulo(num_puntos=2, grados=True, fig=None):
    """
    mide ángulos en la imagen actual a partir de 2 o 3 puntos.
      - Si se usan dos puntos mide el ángulo del vector respecto a x+
      - Si es a 3 puntos (en orden, p1,p2,p3) mide el angulo entre (p2,p1) y (p2,p3).
    
    Parámetros:
        num_puntos: numero de puntos para medir ángulos
        grados: si es True devuelve grados, sino radianes
        fig: figura con UN axis en el que está la imagen
    
    Devuelve:
        angulo
        
    Ejemplo:
        archivo_img = "img_01_micro_mm.jpg"
        img = plt.imread(archivo_img)
        
        fig, ax = plt.subplots( 1,1, figsize=(10,7) ,  constrained_layout=True )
        ax.imshow( img  )
        ax.set_aspect(1)
        ax.set_xlabel('x [px]')
        ax.set_ylabel('y [px]')
        
        ang = medir_angulo()
    """
    if fig is None:
        fig = plt.gcf()
    
    inverted = -1 if fig.axes[0].yaxis.get_inverted() else 1
    
    if num_puntos==2:
        vec_puntos = np.array( fig.ginput(2) ) 
        vector  = np.diff(vec_puntos.T,1).flatten()    
        return np.angle( vector.dot([1,inverted*1j]) , deg=grados)
    elif num_puntos==3:
        vec_puntos = np.array( fig.ginput(3) )
        vec_puntos = (vec_puntos - vec_puntos[1])[0:3:2]
        complex_val = vec_puntos.dot([1,inverted*1j])
        
        return np.angle(complex_val[1]/complex_val[0],deg=grados)
    else:
        raise ValueError("num_puntos debe ser 2 o 3")
    





def rotar_imagen(img, grados=0):
    """
    Rota una imagen en grados
    Devuelve la imagen rotada
    """

    return ndimage.rotate( img , grados)



def alinear_imagen(img, vector=None, horizontal=True, autoclose=True):
    """
    Rota la imagen hasta alinearla con un vector (en forma horizontal o vertical)
    
    Ejemplo:
        archivo_img = "img_01_micro_mm.jpg"
        img = plt.imread(archivo_img)
        
        img2 , vec = alinear_imagen(img, horizontal=False, autoclose=False)
        
        fig, ax = plt.subplots( 1,1, figsize=(10,7) ,  constrained_layout=True )
        ax.imshow( img2  )
        ax.set_aspect(1)
        ax.set_xlabel('x [px]')
        ax.set_ylabel('y [px]')
    """
    if vector is None:
        fig, ax = plt.subplots( 1,1, figsize=(10,7) ,  constrained_layout=True )
        ax.imshow( img  )
        ax.set_aspect(1)
        ax.set_xlabel('x [px]')
        ax.set_ylabel('y [px]')
        vector = np.array( fig.ginput(2) ) 
        
        if autoclose:
            plt.close(fig)
        else:
            ax.plot(vector[:,0],vector[:,1],'-',color='red',lw=3)
        
        #vector[:,1] = -vector[:,1]
        
    if vector.shape == (2,2):
        vector  = np.diff(vector.T,1).flatten()  
        vector  = vector/np.linalg.norm(vector)
    
    angulo = -np.angle( vector.dot([1,-1j]) , deg=True)
    
    if not horizontal:
        angulo += 90
    
    print(f"angulo: {angulo}")
    return rotar_imagen( img, grados=angulo ), vector
        


def recortar_imagen(img, coord=None, autoclose=True):
    """
    Recorta una imagen a partir de las coordenadas dadas.
    Si no se sumunistran coordenadas se abre una evntana para seleccionarlas
    con el mouse.
    
    Devuelve:
        img_recortada , extent
        
    Ejemplo:
        archivo_img = "img_01_micro_mm.jpg"
        img = plt.imread(archivo_img)
        
        img2 , extent = recortar_imagen(img, autoclose=False)
        
        fig, ax = plt.subplots( 1,1, figsize=(10,7) ,  constrained_layout=True )
        ax.imshow( img2 , extent=extent )
        
        ax.set_aspect(1)
        ax.set_xlabel('x [px]')
        ax.set_ylabel('y [px]')
    """
    if coord is None:
        fig, ax = plt.subplots( 1,1, figsize=(10,7) ,  constrained_layout=True )
        ax.imshow( img  )
        ax.set_aspect(1)
        ax.set_xlabel('x [px]')
        ax.set_ylabel('y [px]')
        coord = np.array( fig.ginput(2) )
        x1,x2 = sorted(coord[:,0])
        y1,y2 = sorted(coord[:,1])
        if autoclose:
            plt.close(fig)
        else:
            ax.plot([x1,x1,x2,x2,x1],[y1,y2,y2,y1,y1],'-',color='red',lw=3)
        
    elif hasattr(coord,'shape') and not coord.shape == (2,2):
        raise ValueError("las coordenadas deben ser dos puntos de ma imagen en unidades de píxel")
        
    if type(coord)==list:
        coord = np.array(coord)
    
    
    coord = coord.round().astype(int)
    x1,x2 = sorted(coord[:,0])
    y1,y2 = sorted(coord[:,1])
    
    return img[y1:y2,x1:x2] , [x1,x2,y2,y1]





def perfil_imagen(img, vector=None, autoclose=True, order=1):
    """
    Permite extraer el perfil de intensidades sobre un segmento de una imagen
    Si no se suministra el vector del segmento inicia una GUI para que lo
    seleccione en usuario mediato el mouse.
    
    Ejemplo:
        archivo_img = "img_01_micro_mm.jpg"
        img = plt.imread(archivo_img)

        perfiles , segmento = perfil_imagen(img, autoclose=True)

        fig, ax = plt.subplots( 1,1, figsize=(10,7) ,  constrained_layout=True )
        ax.plot(segmento , perfiles[0] , color='C3')
        ax.plot(segmento , perfiles[1] , color='C2')
        ax.plot(segmento , perfiles[2] , color='C0')
        ax.set_xlabel('segmento [px]')
        ax.set_ylabel('intensidad [ua]')
    """
    if vector is None:
        fig, ax = plt.subplots( 1,1, figsize=(10,7) ,  constrained_layout=True )
        ax.imshow( img  )
        ax.set_aspect(1)
        ax.set_xlabel('x [px]')
        ax.set_ylabel('y [px]')
        vector = np.array( fig.ginput(2) ) 
        
        if autoclose:
            plt.close(fig)
        else:
            ax.plot(vector[:,0],vector[:,1],'-',color='red',lw=3)
            
    num_pts = int(round(np.linalg.norm(vector[0]-vector[1])))
    x1,x2 = sorted(vector[:,0])
    y1,y2 = sorted(vector[:,1])
    xx = np.linspace( x1, x2,  num_pts ).tolist()
    yy = np.linspace( y1, y2,  num_pts ).tolist()
    
    segmento = np.arange( num_pts )
    
    if len(img.shape) == 3:
        perfiles = []
        for ii in range(img.shape[2]):
            perfiles.append(
                    ndimage.map_coordinates(img[:,:,ii], [yy, xx], order=order)
                )
    else:
        perfiles = ndimage.map_coordinates(img, [yy, xx], order=order)
    
    return np.array(perfiles) , segmento






# f = lambda x: savgol(x,81,1)

# fig, ax = plt.subplots( 1,1, figsize=(10,7) ,  constrained_layout=True )
# ax.plot(segmento , f(perfiles[0]) , color='C3')
# ax.plot(segmento , f(perfiles[1]) , color='C2')
# ax.plot(segmento , f(perfiles[2]) , color='C0')
# ax.set_xlabel('segmento [px]')
# ax.set_ylabel('intensidad [ua]')







